package org.osm2world.core.target;

import java.util.Collection;
import java.util.List;

import org.osm2world.core.math.TriangleXYZ;
import org.osm2world.core.math.TriangleXYZWithNormals;
import org.osm2world.core.math.VectorXYZ;

public interface Target {
	
	void drawTriangles(Material material, Collection<TriangleXYZ> triangles);

	void drawTrianglesWithNormals(Material material, Collection<TriangleXYZWithNormals> triangles);

	void drawTriangleStrip(Material material, VectorXYZ... vs); //TODO: delete this, use only collections
	
	void drawTriangleStrip(Material material, List<? extends VectorXYZ> vs);
	
	void drawTriangleFan(Material material, List<? extends VectorXYZ> vs);
	
	void drawPolygon(Material material, VectorXYZ... vs);

	/**
	 * draws a box with outward-facing polygons
	 */
	void drawBox(Material material, VectorXYZ frontLowerLeft, 
			VectorXYZ rightVector, VectorXYZ upVector, VectorXYZ backVector);

	/**
	 * draws a column with outward-facing polygons around a point. 
	 * A column is a polygon with > 3 corners extruded upwards.
	 * 
	 * The implementation may decide to reduce the number of corners
	 * in order to improve performance (or make rendering possible
	 * when a perfect cylinder isn't supported).
	 * @param corners  number of corners; null creates a cylinder
	 *  for radiusBottom == radiusTop or (truncated) cone otherwise
	 */
	void drawColumn(Material material, Integer corners, 
			VectorXYZ base, double height, double radiusBottom,
			double radiusTop, boolean drawBottom, boolean drawTop);
	
}
