package org.osm2world.core.world.modules.common;

import java.awt.Color;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.osm2world.core.target.Material;
import org.osm2world.core.target.Material.Lighting;
import org.osm2world.core.world.creation.WorldModule;

/**
 * this class defines materials that can be used by all {@link WorldModule}s
 */
public final class Materials {

	/** prevents instantiation */
	private Materials() {}

	/** material for "empty" ground */
	public static final Material EMPTY_GROUND = 
		new Material(Lighting.SMOOTH, Color.GREEN);

	public static final Material WATER = 
		new Material(Lighting.FLAT, Color.BLUE);

	public static final Material ASPHALT = 
		new Material(Lighting.FLAT, new Color(0.3f, 0.3f, 0.3f));
	public static final Material EARTH =
		new Material(Lighting.FLAT, new Color( 0.3f, 0, 0));
	public static final Material GRASS = 
		new Material(Lighting.FLAT, new Color(0.0f, 0.8f, 0.0f));
	public static final Material GRAVEL = 
		new Material(Lighting.FLAT, new Color(0.4f, 0.4f, 0.4f));
	public static final Material SAND = 
		new Material(Lighting.FLAT, new Color(241, 233, 80));
	public static final Material WOOD = 
		new Material(Lighting.FLAT, new Color(0.3f, 0.2f, 0.2f));
	public static final Material TARTAN =
		new Material(Lighting.FLAT, new Color(206, 109, 90));

	private static final Map<String, Material> materialMap = new HashMap<String, Material>();

	static {
		materialMap.put("asphalt", ASPHALT);
		materialMap.put("cobblestone", ASPHALT);
		materialMap.put("compacted", GRAVEL);
		materialMap.put("concrete", ASPHALT);
		materialMap.put("grass", GRASS);
		materialMap.put("gravel", GRAVEL);
		materialMap.put("grass_paver", ASPHALT);
		materialMap.put("ground", EARTH);
		materialMap.put("paved", ASPHALT);
		materialMap.put("paving_stones", ASPHALT);
		materialMap.put("pebblestone", ASPHALT);
		materialMap.put("sand", SAND);
		materialMap.put("tartan", TARTAN);
		materialMap.put("unpaved", EARTH);
		materialMap.put("wood", WOOD);
	}

	/** returns all materials defined here */
	public static final Collection<Material> getMaterials() {
		return materialMap.values();
	}
	
	/** returns a material for a surface value; null if none is found */
	public static final Material getMaterial(String value) {
		return getMaterial(value, null);
	}
	
	/** same as {@link #getMaterial(String)}, but with fallback value */
	public static final Material getMaterial(String value, Material fallback) {
		Material material = materialMap.get(value);
		if (material != null) {
			return material;
		} else {
			return fallback;
		}
	}
	
}
