package org.osm2world.viewer.control.actions;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.osm2world.viewer.model.Data;
import org.osm2world.viewer.model.MessageManager;
import org.osm2world.viewer.model.RenderOptions;
import org.osm2world.viewer.view.ViewerFrame;


public abstract class AbstractExportAction extends AbstractAction {

	protected final ViewerFrame viewerFrame;
	protected final Data data;
	protected final MessageManager messageManager;
	protected final RenderOptions renderOptions;
	
	private File lastPath = null;

	protected AbstractExportAction(String name,
			ViewerFrame viewerFrame, Data data,
			MessageManager messageManager,
			RenderOptions renderOptions) {
		
		super(name);
		
		this.viewerFrame = viewerFrame;
		this.messageManager = messageManager;
		this.data = data;
		this.renderOptions = renderOptions;
		
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
	
		File file = askFile();
	
		if (file != null) {
			
			viewerFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	
			performExport(file);
			
			viewerFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			
		}
		
	}
	
	private File askFile() {
		
		JFileChooser chooser = new JFileChooser(lastPath);
		chooser.setDialogTitle("Export file");
		chooser.setFileFilter(getFileNameExtensionFilter());
		
		int returnVal = chooser.showSaveDialog(null);
	
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			
			File selectedFile = chooser.getSelectedFile();
			
			lastPath = selectedFile.getParentFile();
			
			/* make sure that file uses correct extension */
			
			if (getFileNameExtensionFilter().getExtensions().length == 1) {
				
				String extension = getFileNameExtensionFilter().getExtensions()[0];
								
				if (!selectedFile.getPath().endsWith("." + extension)) {
					selectedFile = new File(selectedFile.getPath() + "." + extension);
				}
				
			}
			
			return selectedFile;
			
		} else {
			return null;
		}
		
	}

	abstract protected FileNameExtensionFilter getFileNameExtensionFilter();

	abstract protected void performExport(File file);
	
}
